/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_ProfileTimelineMarkerBinding_h
#define mozilla_dom_ProfileTimelineMarkerBinding_h

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

struct NativePropertyHooks;
struct ProfileTimelineLayerRect;
struct ProfileTimelineLayerRectAtoms;
struct ProfileTimelineMarkerAtoms;
class ProtoAndIfaceCache;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

struct ProfileTimelineLayerRect : public DictionaryBase
{
  int32_t mHeight;
  int32_t mWidth;
  int32_t mX;
  int32_t mY;

  ProfileTimelineLayerRect();

  explicit inline ProfileTimelineLayerRect(const ProfileTimelineLayerRect& aOther)
  {
    *this = aOther;
  }

protected:
  explicit inline ProfileTimelineLayerRect(int )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

private:
  static bool
  InitIds(JSContext* cx, ProfileTimelineLayerRectAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value", bool passedToJSImpl = false);

  bool
  Init(const nsAString& aJSON);

  bool
  ToObjectInternal(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  bool
  ToJSON(nsAString& aJSON) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const ProfileTimelineLayerRect& aOther);
};

namespace binding_detail {
struct FastProfileTimelineLayerRect : public ProfileTimelineLayerRect
{
  inline FastProfileTimelineLayerRect()
    : ProfileTimelineLayerRect(42)
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


struct ProfileTimelineMarker : public DictionaryBase
{
  Optional<nsString> mCauseName;
  double mEnd;
  JSObject* mEndStack;
  Optional<uint16_t> mEventPhase;
  nsString mName;
  Optional<Sequence<ProfileTimelineLayerRect>> mRectangles;
  JSObject* mStack;
  double mStart;
  Optional<nsString> mType;

  ProfileTimelineMarker();

protected:
  explicit inline ProfileTimelineMarker(int )
    : mEndStack(nullptr),
      mStack(nullptr)
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

private:
  ProfileTimelineMarker(const ProfileTimelineMarker&) = delete;
  void operator=(const ProfileTimelineMarker) = delete;

  static bool
  InitIds(JSContext* cx, ProfileTimelineMarkerAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value", bool passedToJSImpl = false);

  bool
  Init(const nsAString& aJSON);

  bool
  ToObjectInternal(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  bool
  ToJSON(nsAString& aJSON) const;

  void
  TraceDictionary(JSTracer* trc);
};

namespace binding_detail {
struct FastProfileTimelineMarker : public ProfileTimelineMarker
{
  inline FastProfileTimelineMarker()
    : ProfileTimelineMarker(42)
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_ProfileTimelineMarkerBinding_h
