/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/mozIGeckoMediaPluginService.idl
 */

#ifndef __gen_mozIGeckoMediaPluginService_h__
#define __gen_mozIGeckoMediaPluginService_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

#ifndef __gen_nsIThread_h__
#include "nsIThread.h"
#endif

#ifndef __gen_nsIPrincipal_h__
#include "nsIPrincipal.h"
#endif

#ifndef __gen_nsIFile_h__
#include "nsIFile.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
#include "nsTArray.h"
#include "nsStringGlue.h"
class GMPAudioDecoderProxy;
class GMPDecryptorProxy;
class GMPVideoDecoderProxy;
class GMPVideoEncoderProxy;
class GMPVideoHost;

/* starting interface:    mozIGeckoMediaPluginService */
#define MOZIGECKOMEDIAPLUGINSERVICE_IID_STR "4aaf58d3-181f-4325-9a2f-41172d995a70"

#define MOZIGECKOMEDIAPLUGINSERVICE_IID \
  {0x4aaf58d3, 0x181f, 0x4325, \
    { 0x9a, 0x2f, 0x41, 0x17, 0x2d, 0x99, 0x5a, 0x70 }}

class NS_NO_VTABLE mozIGeckoMediaPluginService : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(MOZIGECKOMEDIAPLUGINSERVICE_IID)

  /* readonly attribute nsIThread thread; */
  NS_IMETHOD GetThread(nsIThread * *aThread) = 0;

  /* [noscript] boolean hasPluginForAPI (in ACString api, in TagArray tags); */
  NS_IMETHOD HasPluginForAPI(const nsACString & api, nsTArray<nsCString> *tags, bool *_retval) = 0;

  /* [noscript] GMPVideoDecoderProxy getGMPVideoDecoder (in TagArray tags, [optional] in ACString nodeId, out GMPVideoHost outVideoHost); */
  NS_IMETHOD GetGMPVideoDecoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPVideoHost **outVideoHost, GMPVideoDecoderProxy **_retval) = 0;

  /* [noscript] GMPVideoEncoderProxy getGMPVideoEncoder (in TagArray tags, [optional] in ACString nodeId, out GMPVideoHost outVideoHost); */
  NS_IMETHOD GetGMPVideoEncoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPVideoHost **outVideoHost, GMPVideoEncoderProxy **_retval) = 0;

  /* GMPAudioDecoderProxy getGMPAudioDecoder (in TagArray tags, [optional] in ACString nodeId); */
  NS_IMETHOD GetGMPAudioDecoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPAudioDecoderProxy **_retval) = 0;

  /* GMPDecryptorProxy getGMPDecryptor (in TagArray tags, in ACString nodeId); */
  NS_IMETHOD GetGMPDecryptor(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPDecryptorProxy **_retval) = 0;

  /* void addPluginDirectory (in AString directory); */
  NS_IMETHOD AddPluginDirectory(const nsAString & directory) = 0;

  /* void removePluginDirectory (in AString directory); */
  NS_IMETHOD RemovePluginDirectory(const nsAString & directory) = 0;

  /* ACString getNodeId (in AString origin, in AString topLevelOrigin, in bool inPrivateBrowsingMode); */
  NS_IMETHOD GetNodeId(const nsAString & origin, const nsAString & topLevelOrigin, bool inPrivateBrowsingMode, nsACString & _retval) = 0;

  /* void forgetThisSite (in AString origin); */
  NS_IMETHOD ForgetThisSite(const nsAString & origin) = 0;

  /* bool isPersistentStorageAllowed (in ACString nodeId); */
  NS_IMETHOD IsPersistentStorageAllowed(const nsACString & nodeId, bool *_retval) = 0;

  /* nsIFile getStorageDir (); */
  NS_IMETHOD GetStorageDir(nsIFile * *_retval) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(mozIGeckoMediaPluginService, MOZIGECKOMEDIAPLUGINSERVICE_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_MOZIGECKOMEDIAPLUGINSERVICE \
  NS_IMETHOD GetThread(nsIThread * *aThread) MOZ_OVERRIDE; \
  NS_IMETHOD HasPluginForAPI(const nsACString & api, nsTArray<nsCString> *tags, bool *_retval) MOZ_OVERRIDE; \
  NS_IMETHOD GetGMPVideoDecoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPVideoHost **outVideoHost, GMPVideoDecoderProxy **_retval) MOZ_OVERRIDE; \
  NS_IMETHOD GetGMPVideoEncoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPVideoHost **outVideoHost, GMPVideoEncoderProxy **_retval) MOZ_OVERRIDE; \
  NS_IMETHOD GetGMPAudioDecoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPAudioDecoderProxy **_retval) MOZ_OVERRIDE; \
  NS_IMETHOD GetGMPDecryptor(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPDecryptorProxy **_retval) MOZ_OVERRIDE; \
  NS_IMETHOD AddPluginDirectory(const nsAString & directory) MOZ_OVERRIDE; \
  NS_IMETHOD RemovePluginDirectory(const nsAString & directory) MOZ_OVERRIDE; \
  NS_IMETHOD GetNodeId(const nsAString & origin, const nsAString & topLevelOrigin, bool inPrivateBrowsingMode, nsACString & _retval) MOZ_OVERRIDE; \
  NS_IMETHOD ForgetThisSite(const nsAString & origin) MOZ_OVERRIDE; \
  NS_IMETHOD IsPersistentStorageAllowed(const nsACString & nodeId, bool *_retval) MOZ_OVERRIDE; \
  NS_IMETHOD GetStorageDir(nsIFile * *_retval) MOZ_OVERRIDE; 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_MOZIGECKOMEDIAPLUGINSERVICE(_to) \
  NS_IMETHOD GetThread(nsIThread * *aThread) MOZ_OVERRIDE { return _to GetThread(aThread); } \
  NS_IMETHOD HasPluginForAPI(const nsACString & api, nsTArray<nsCString> *tags, bool *_retval) MOZ_OVERRIDE { return _to HasPluginForAPI(api, tags, _retval); } \
  NS_IMETHOD GetGMPVideoDecoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPVideoHost **outVideoHost, GMPVideoDecoderProxy **_retval) MOZ_OVERRIDE { return _to GetGMPVideoDecoder(tags, nodeId, outVideoHost, _retval); } \
  NS_IMETHOD GetGMPVideoEncoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPVideoHost **outVideoHost, GMPVideoEncoderProxy **_retval) MOZ_OVERRIDE { return _to GetGMPVideoEncoder(tags, nodeId, outVideoHost, _retval); } \
  NS_IMETHOD GetGMPAudioDecoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPAudioDecoderProxy **_retval) MOZ_OVERRIDE { return _to GetGMPAudioDecoder(tags, nodeId, _retval); } \
  NS_IMETHOD GetGMPDecryptor(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPDecryptorProxy **_retval) MOZ_OVERRIDE { return _to GetGMPDecryptor(tags, nodeId, _retval); } \
  NS_IMETHOD AddPluginDirectory(const nsAString & directory) MOZ_OVERRIDE { return _to AddPluginDirectory(directory); } \
  NS_IMETHOD RemovePluginDirectory(const nsAString & directory) MOZ_OVERRIDE { return _to RemovePluginDirectory(directory); } \
  NS_IMETHOD GetNodeId(const nsAString & origin, const nsAString & topLevelOrigin, bool inPrivateBrowsingMode, nsACString & _retval) MOZ_OVERRIDE { return _to GetNodeId(origin, topLevelOrigin, inPrivateBrowsingMode, _retval); } \
  NS_IMETHOD ForgetThisSite(const nsAString & origin) MOZ_OVERRIDE { return _to ForgetThisSite(origin); } \
  NS_IMETHOD IsPersistentStorageAllowed(const nsACString & nodeId, bool *_retval) MOZ_OVERRIDE { return _to IsPersistentStorageAllowed(nodeId, _retval); } \
  NS_IMETHOD GetStorageDir(nsIFile * *_retval) MOZ_OVERRIDE { return _to GetStorageDir(_retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_MOZIGECKOMEDIAPLUGINSERVICE(_to) \
  NS_IMETHOD GetThread(nsIThread * *aThread) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->GetThread(aThread); } \
  NS_IMETHOD HasPluginForAPI(const nsACString & api, nsTArray<nsCString> *tags, bool *_retval) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->HasPluginForAPI(api, tags, _retval); } \
  NS_IMETHOD GetGMPVideoDecoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPVideoHost **outVideoHost, GMPVideoDecoderProxy **_retval) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->GetGMPVideoDecoder(tags, nodeId, outVideoHost, _retval); } \
  NS_IMETHOD GetGMPVideoEncoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPVideoHost **outVideoHost, GMPVideoEncoderProxy **_retval) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->GetGMPVideoEncoder(tags, nodeId, outVideoHost, _retval); } \
  NS_IMETHOD GetGMPAudioDecoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPAudioDecoderProxy **_retval) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->GetGMPAudioDecoder(tags, nodeId, _retval); } \
  NS_IMETHOD GetGMPDecryptor(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPDecryptorProxy **_retval) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->GetGMPDecryptor(tags, nodeId, _retval); } \
  NS_IMETHOD AddPluginDirectory(const nsAString & directory) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->AddPluginDirectory(directory); } \
  NS_IMETHOD RemovePluginDirectory(const nsAString & directory) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->RemovePluginDirectory(directory); } \
  NS_IMETHOD GetNodeId(const nsAString & origin, const nsAString & topLevelOrigin, bool inPrivateBrowsingMode, nsACString & _retval) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->GetNodeId(origin, topLevelOrigin, inPrivateBrowsingMode, _retval); } \
  NS_IMETHOD ForgetThisSite(const nsAString & origin) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->ForgetThisSite(origin); } \
  NS_IMETHOD IsPersistentStorageAllowed(const nsACString & nodeId, bool *_retval) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->IsPersistentStorageAllowed(nodeId, _retval); } \
  NS_IMETHOD GetStorageDir(nsIFile * *_retval) MOZ_OVERRIDE { return !_to ? NS_ERROR_NULL_POINTER : _to->GetStorageDir(_retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class _MYCLASS_ : public mozIGeckoMediaPluginService
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_MOZIGECKOMEDIAPLUGINSERVICE

  _MYCLASS_();

private:
  ~_MYCLASS_();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(_MYCLASS_, mozIGeckoMediaPluginService)

_MYCLASS_::_MYCLASS_()
{
  /* member initializers and constructor code */
}

_MYCLASS_::~_MYCLASS_()
{
  /* destructor code */
}

/* readonly attribute nsIThread thread; */
NS_IMETHODIMP _MYCLASS_::GetThread(nsIThread * *aThread)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [noscript] boolean hasPluginForAPI (in ACString api, in TagArray tags); */
NS_IMETHODIMP _MYCLASS_::HasPluginForAPI(const nsACString & api, nsTArray<nsCString> *tags, bool *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [noscript] GMPVideoDecoderProxy getGMPVideoDecoder (in TagArray tags, [optional] in ACString nodeId, out GMPVideoHost outVideoHost); */
NS_IMETHODIMP _MYCLASS_::GetGMPVideoDecoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPVideoHost **outVideoHost, GMPVideoDecoderProxy **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [noscript] GMPVideoEncoderProxy getGMPVideoEncoder (in TagArray tags, [optional] in ACString nodeId, out GMPVideoHost outVideoHost); */
NS_IMETHODIMP _MYCLASS_::GetGMPVideoEncoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPVideoHost **outVideoHost, GMPVideoEncoderProxy **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* GMPAudioDecoderProxy getGMPAudioDecoder (in TagArray tags, [optional] in ACString nodeId); */
NS_IMETHODIMP _MYCLASS_::GetGMPAudioDecoder(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPAudioDecoderProxy **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* GMPDecryptorProxy getGMPDecryptor (in TagArray tags, in ACString nodeId); */
NS_IMETHODIMP _MYCLASS_::GetGMPDecryptor(nsTArray<nsCString> *tags, const nsACString & nodeId, GMPDecryptorProxy **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void addPluginDirectory (in AString directory); */
NS_IMETHODIMP _MYCLASS_::AddPluginDirectory(const nsAString & directory)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void removePluginDirectory (in AString directory); */
NS_IMETHODIMP _MYCLASS_::RemovePluginDirectory(const nsAString & directory)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* ACString getNodeId (in AString origin, in AString topLevelOrigin, in bool inPrivateBrowsingMode); */
NS_IMETHODIMP _MYCLASS_::GetNodeId(const nsAString & origin, const nsAString & topLevelOrigin, bool inPrivateBrowsingMode, nsACString & _retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void forgetThisSite (in AString origin); */
NS_IMETHODIMP _MYCLASS_::ForgetThisSite(const nsAString & origin)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* bool isPersistentStorageAllowed (in ACString nodeId); */
NS_IMETHODIMP _MYCLASS_::IsPersistentStorageAllowed(const nsACString & nodeId, bool *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsIFile getStorageDir (); */
NS_IMETHODIMP _MYCLASS_::GetStorageDir(nsIFile * *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_mozIGeckoMediaPluginService_h__ */
